/* slurp-download-resource.c
 *
 * Copyright © 2017 Julian Richen <julian@richen.io>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#define G_LOG_DOMAIN "slurp-download-resource"

#include "slurp-download-resource.h"

G_DEFINE_BOXED_TYPE (SlurpDownloadResource, slurp_download_resource, slurp_download_resource_ref, slurp_download_resource_unref)

SlurpDownloadResource *
slurp_download_resource_new (void)
{
    SlurpDownloadResource *self;

    self = g_slice_new0 (SlurpDownloadResource);
    self->ref_count = 1;

    self->loop = NULL;

    self->uri = NULL;
    self->path = NULL;
    self->overwrite = FALSE;

    self->cancellable = NULL;
    self->p_handler = NULL;
    self->p_user_data = NULL;

    self->file = NULL;
    self->total_bytes = 0;
    self->downloaded_bytes = 0;
    self->last_progress_time = 0;

    return self;
}

SlurpDownloadResource *
slurp_download_resource_copy (SlurpDownloadResource *self)
{
    SlurpDownloadResource *copy;

    g_return_val_if_fail (self, NULL);
    g_return_val_if_fail (self->ref_count, NULL);

    copy = slurp_download_resource_new ();

    return copy;
}

static void
slurp_download_resource_free (SlurpDownloadResource *self)
{
    g_assert (self);
    g_assert_cmpint (self->ref_count, ==, 0);

    g_main_loop_quit (self->loop);

    g_cancellable_cancel (self->cancellable);
    g_clear_pointer (&self->cancellable, g_object_unref);

    g_clear_pointer (&self->file, g_object_unref);

    if (self)
        g_slice_free (SlurpDownloadResource, self);
}

SlurpDownloadResource *
slurp_download_resource_ref (SlurpDownloadResource *self)
{
    g_return_val_if_fail (self, NULL);
    g_return_val_if_fail (self->ref_count, NULL);

    g_atomic_int_inc (&self->ref_count);

    return self;
}

void
slurp_download_resource_unref (SlurpDownloadResource *self)
{
    g_return_if_fail (self);
    g_return_if_fail (self->ref_count);

    if (g_atomic_int_dec_and_test (&self->ref_count))
        slurp_download_resource_free (self);
}
