/* slurp-download-resource.h
 *
 * Copyright © 2017 Julian Richen <julian@richen.io>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef SLURP_DOWNLOAD_RESOURCE_H
#define SLURP_DOWNLOAD_RESOURCE_H

#include <glib-object.h>
#include <glib.h>
#include <gio/gio.h>

G_BEGIN_DECLS

#define SLURP_TYPE_DOWNLOAD_RESOURCE (slurp_download_resource_get_type())

typedef struct _SlurpDownloadResource SlurpDownloadResource;

struct _SlurpDownloadResource
{
    guint ref_count;

    GMainLoop *loop;

    const gchar *uri;
    const gchar *path;
    gboolean overwrite;

    GCancellable *cancellable;
    GCallback p_handler;
    gpointer p_user_data;

    GFile *file;
    gchar buffer[16384]; // 16 * 1024
    GOutputStream *output;

    guint64 total_bytes;
    guint64 downloaded_bytes;
    guint64 last_progress_time;
};

SlurpDownloadResource *
slurp_download_resource_new (void);

SlurpDownloadResource *
slurp_download_resource_copy (SlurpDownloadResource *self);

SlurpDownloadResource *
slurp_download_resource_ref (SlurpDownloadResource *self);

void
slurp_download_resource_unref (SlurpDownloadResource *self);

G_DEFINE_AUTOPTR_CLEANUP_FUNC (SlurpDownloadResource, slurp_download_resource_unref)

G_END_DECLS

#endif /* SLURP_DOWNLOAD_RESOURCE_H */

