/* slurp-download.h
 *
 * Copyright © 2017 Julian Richen <julian@richen.io>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef SLURP_DOWNLOAD_H
#define SLURP_DOWNLOAD_H

#include "slurp-resource.h"

#include <glib-object.h>
#include <glib.h>
#include <gio/gio.h>
#include <libsoup/soup.h>

G_BEGIN_DECLS

#define SLURP_TYPE_DOWNLOAD (slurp_download_get_type())

G_DECLARE_FINAL_TYPE (SlurpDownload, slurp_download, SLURP, DOWNLOAD, GObject)

struct _SlurpDownload
{
    GObject parent_instance;
};

SlurpDownload *
slurp_download_new (void);

SlurpDownload *
slurp_download_new_with_session (SoupSession *session);

gboolean
slurp_download_uri (SlurpDownload *self,
                    const gchar   *uri);

gboolean
slurp_download_uri_full (SlurpDownload *self,
                         const gchar   *uri,
                         const gchar   *destination,
                         gboolean       overwrite);

void
slurp_download_uri_async (SlurpDownload       *self,
                          const gchar         *uri,
                          GCancellable        *cancellable,
                          GAsyncReadyCallback  c_handler,
                          gpointer             user_data);

void
slurp_download_uri_async_full (SlurpDownload       *self,
                               const gchar         *uri,
                               const gchar         *destination,
                               gboolean             overwrite,
                               GCancellable        *cancellable,
                               GAsyncReadyCallback  c_handler,
                               gpointer             user_data);

G_END_DECLS

#endif /* SLURP_DOWNLOAD_H */

